'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var buffer = require('buffer');
var reactNative = require('react-native');
var utility = require('@react-native-module/utility');
var base64Js = require('base64-js');
var async = require('../pbkdf2Browser/async.js');
var sync = require('../pbkdf2Browser/sync.js');

let doWarn = false;
function arrayBufferViewToArrayBuffer(arrayBufferView) {
    return arrayBufferView.buffer.slice(arrayBufferView.byteOffset, arrayBufferView.byteOffset + arrayBufferView.byteLength);
}
function binaryLikeToBuffer(binaryLike) {
    if (typeof binaryLike === 'string')
        return buffer.Buffer.from(binaryLike, 'utf-8');
    const arrayBuffer = arrayBufferViewToArrayBuffer(binaryLike);
    return Buffer.from(arrayBuffer);
}
function binaryLikeToBase64(binaryLike) {
    if (typeof binaryLike === 'string')
        return buffer.Buffer.from(binaryLike, 'utf-8').toString('base64');
    const arrayBuffer = arrayBufferViewToArrayBuffer(binaryLike);
    const base64 = base64Js.fromByteArray(new Uint8Array(arrayBuffer));
    return base64;
}
function warnUnsupport() {
    if (reactNative.Platform.OS === 'ios' && doWarn === false) {
        doWarn = true;
        console.warn('@react-native-module/pbkdf2 not support on IOS yet');
    }
}
function isSupport() {
    return utility.Environment === 'NativeMobile' && reactNative.Platform.OS === 'android';
}
function canUseNativeModule(functionName) {
    try {
        return Boolean(reactNative.NativeModules.Pbkdf2[functionName]);
    }
    catch (error) {
        return false;
    }
}
// Node API from @types/node
// function pbkdf2(password: BinaryLike, salt: BinaryLike, iterations: number, keylen: number, digest: string, callback: (err: Error | null, derivedKey: Buffer) => void): void;
function pbkdf2(password, salt, iterations, keylen, digest = 'sha1', callback) {
    if (isSupport() && canUseNativeModule('derive')) {
        reactNative.NativeModules.Pbkdf2.derive(binaryLikeToBase64(password), binaryLikeToBase64(salt), iterations, keylen, digest)
            .then((base64Result) => {
            callback(null, buffer.Buffer.from(base64Result, 'base64'));
        })
            .catch((error) => {
            if (error instanceof Error || error === null) {
                callback(error, buffer.Buffer.alloc(0));
            }
        });
    }
    else {
        warnUnsupport();
        try {
            async.async(binaryLikeToBuffer(password), binaryLikeToBuffer(salt), iterations, keylen, digest, callback);
        }
        catch (error) {
            if (error instanceof Error || error === null) {
                callback(error, buffer.Buffer.alloc(0));
            }
        }
    }
}
// function pbkdf2Sync(password: BinaryLike, salt: BinaryLike, iterations: number, keylen: number, digest: string): Buffer;
function pbkdf2Sync(password, salt, iterations, keylen, digest) {
    if (isSupport() && canUseNativeModule('deriveSync')) {
        const base64Result = reactNative.NativeModules.Pbkdf2.deriveSync(binaryLikeToBase64(password), binaryLikeToBase64(salt), iterations, keylen, digest);
        return buffer.Buffer.from(base64Result, 'base64');
    }
    else {
        warnUnsupport();
        return sync.sync(binaryLikeToBuffer(password), binaryLikeToBuffer(salt), iterations, keylen, digest);
    }
}

exports.binaryLikeToBase64 = binaryLikeToBase64;
exports.pbkdf2 = pbkdf2;
exports.pbkdf2Sync = pbkdf2Sync;
