import { Buffer as Buffer$1 } from 'buffer';
import { NativeModules, Platform } from 'react-native';
import { Environment } from '@react-native-module/utility';
import { fromByteArray } from 'base64-js';
import { async } from '../pbkdf2Browser/async.js';
import { sync } from '../pbkdf2Browser/sync.js';

let doWarn = false;
function arrayBufferViewToArrayBuffer(arrayBufferView) {
    return arrayBufferView.buffer.slice(arrayBufferView.byteOffset, arrayBufferView.byteOffset + arrayBufferView.byteLength);
}
function binaryLikeToBuffer(binaryLike) {
    if (typeof binaryLike === 'string')
        return Buffer$1.from(binaryLike, 'utf-8');
    const arrayBuffer = arrayBufferViewToArrayBuffer(binaryLike);
    return Buffer.from(arrayBuffer);
}
function binaryLikeToBase64(binaryLike) {
    if (typeof binaryLike === 'string')
        return Buffer$1.from(binaryLike, 'utf-8').toString('base64');
    const arrayBuffer = arrayBufferViewToArrayBuffer(binaryLike);
    const base64 = fromByteArray(new Uint8Array(arrayBuffer));
    return base64;
}
function warnUnsupport() {
    if (Platform.OS === 'ios' && doWarn === false) {
        doWarn = true;
        console.warn('@react-native-module/pbkdf2 not support on IOS yet');
    }
}
function isSupport() {
    return Environment === 'NativeMobile' && Platform.OS === 'android';
}
function canUseNativeModule(functionName) {
    try {
        return Boolean(NativeModules.Pbkdf2[functionName]);
    }
    catch (error) {
        return false;
    }
}
// Node API from @types/node
// function pbkdf2(password: BinaryLike, salt: BinaryLike, iterations: number, keylen: number, digest: string, callback: (err: Error | null, derivedKey: Buffer) => void): void;
function pbkdf2(password, salt, iterations, keylen, digest = 'sha1', callback) {
    if (isSupport() && canUseNativeModule('derive')) {
        NativeModules.Pbkdf2.derive(binaryLikeToBase64(password), binaryLikeToBase64(salt), iterations, keylen, digest)
            .then((base64Result) => {
            callback(null, Buffer$1.from(base64Result, 'base64'));
        })
            .catch((error) => {
            if (error instanceof Error || error === null) {
                callback(error, Buffer$1.alloc(0));
            }
        });
    }
    else {
        warnUnsupport();
        try {
            async(binaryLikeToBuffer(password), binaryLikeToBuffer(salt), iterations, keylen, digest, callback);
        }
        catch (error) {
            if (error instanceof Error || error === null) {
                callback(error, Buffer$1.alloc(0));
            }
        }
    }
}
// function pbkdf2Sync(password: BinaryLike, salt: BinaryLike, iterations: number, keylen: number, digest: string): Buffer;
function pbkdf2Sync(password, salt, iterations, keylen, digest) {
    if (isSupport() && canUseNativeModule('deriveSync')) {
        const base64Result = NativeModules.Pbkdf2.deriveSync(binaryLikeToBase64(password), binaryLikeToBase64(salt), iterations, keylen, digest);
        return Buffer$1.from(base64Result, 'base64');
    }
    else {
        warnUnsupport();
        return sync(binaryLikeToBuffer(password), binaryLikeToBuffer(salt), iterations, keylen, digest);
    }
}

export { binaryLikeToBase64, pbkdf2, pbkdf2Sync };
